/*!
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: tcps.c,v $
 * Revision 1.7  2002/06/26 17:29:06  harald
 * First pre-release with 2.4 stack
 *
 * Revision 1.6  2002/06/12 10:59:05  harald
 * *** empty log message ***
 *
 * Revision 1.5  2002/06/04 19:12:55  harald
 * *** empty log message ***
 *
 * Revision 1.4  2002/05/08 16:02:32  harald
 * First Imagecraft compilation
 *
 */

/*!
 * \example tcps/tcps.c
 *
 * Simple TCP server.
 *
 * Program Ethernut with tcps.rom and enter
 *
 * telnet x.x.x.x 12345
 *
 * on a command prompt, replacing x.x.x.x with the
 * IP address of your ethernut board. Enter help
 * for a list of available commands.
 */

#include <string.h>

#include <dev/nicrtl.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>

static u_char buff[128];

/*
 * Process client requests.
 */
void ProcessRequests(NUTDEVICE *sostream)
{
    int l;

    strcmp(buff, "TEST");
    NutPrintString(sostream, "200 Welcome to tcps. Type help to get help.\r\n");
    for(;;) {
        NutPrintFlush(sostream);
        if(NutDeviceGetLine(sostream, buff, sizeof(buff)) < 0)
            break;
        /*
         * Ignore blank lines.
         */
        if(buff[0] == 0)
            continue;

        /*
         * Memory info.
         */
        l = strlen(buff);
        if(strncmp(buff, "memory", l) == 0) {
            NutPrintFormat(sostream, "210 %u bytes RAM free\r\n", NutHeapAvailable());
            continue;
        }

        /*
         * List threads.
         */
        if(strncmp(buff, "threads", l) == 0) {
            NUTTHREADINFO *tdp;
            NUTTIMERINFO *tnp;

            NutPrintString(sostream, "220 List of threads with name,state,prio,stack,mem,timeout follows\r\n");
            for(tdp = nutThreadList; tdp; tdp = tdp->td_next) {
                NutPrintString(sostream, tdp->td_name);
                NutPrintString(sostream, "\t");
                switch(tdp->td_state) {
                case TDS_TERM:
                    NutPrintString(sostream, "\tTerm\t");
                    break;
                case TDS_RUNNING:
                    NutPrintString(sostream, "\tRun\t");
                    break;
                case TDS_READY:
                    NutPrintString(sostream, "\tReady\t");
                    break;
                case TDS_SLEEP:
                    NutPrintString(sostream, "\tSleep\t");
                    break;
                }
                NutPrintInteger(sostream, tdp->td_priority, 10, 0, 0);
                NutPrintString(sostream, "\t");
                NutPrintInteger(sostream, (u_short)tdp->td_sp - (u_short)tdp->td_memory, 10, 0, 0);
                if(*((u_long *)tdp->td_memory) != DEADBEEF)
                    NutPrintString(sostream, "\tCorrupted\t");
                else
                    NutPrintString(sostream, "\tOK\t");

                if((tnp = (NUTTIMERINFO *)tdp->td_timer) != 0) 
                    NutPrintInteger(sostream, tnp->tn_ticks_left, 10, 0, 0);
                else
                    NutPrintString(sostream, "None");
                NutPrintString(sostream, "\r\n");
            }
            NutPrintString(sostream, ".\r\n");
            continue;
        }

        /*
         * List threads.
         */
        if(strncmp("timers", buff, l) == 0) {
            NUTTIMERINFO *tnp;

            NutPrintString(sostream, "221 List of timers with ticks left and interval follows\r\n");
            for(tnp = nutTimerList; tnp; tnp = tnp->tn_next) {
                NutPrintInteger(sostream, tnp->tn_ticks_left, 10, 0, 0);
                if(tnp->tn_ticks) {
                    NutPrintString(sostream, "\t");
                    NutPrintInteger(sostream, tnp->tn_ticks, 10, 0, 0);
                }
                else 
                    NutPrintString(sostream, "\tOneshot");
                NutPrintString(sostream, "\r\n");
            }
            NutPrintString(sostream, ".\r\n");
            continue;
        }
        NutPrintString(sostream, "400 List of commands follows\r\n");
        NutPrintString(sostream, "memory\tQueries number of RAM bytes free\r\n");
        NutPrintString(sostream, "threads\tLists all created threads\r\n");
        NutPrintString(sostream, "timers\tLists all running timers\r\n");
        NutPrintString(sostream, ".\r\n");
    }
}

/*
 * Main application routine. 
 *
 * Nut/OS automatically calls this entry after initialization.
 */
THREAD(NutMain, arg)
{
    TCPSOCKET *sock;
    NUTDEVICE *sostream;

    /*
     * Register Realtek controller at address 8300 hex
     * and interrupt 5.
     */
    NutRegisterDevice(&devEth0, 0x8300, 5);

    /*
     * Configure lan interface. 
     *
     */
    NutNetAutoConfig("eth0");
    
    /*
     * Now loop endless for connections.
     */
    for(;;) {
        /*
         * Create a socket.
         */
        sock = NutTcpCreateSocket();

        /*
         * Listen on port 12345. If we return,
         * we got a client.
         */
        NutTcpAccept(sock, 12345);

        /*
         * Create a device from the socket,
         * so we can use NutPrintFormat() and
         * other device goodies.
         */
        sostream = NutSoStreamCreate(sock);

        /*
         * Process client requests.
         */
        ProcessRequests(sostream);

        /*
         * Destroy our device.
         */
        NutSoStreamDestroy(sostream);

        /*
         * Close our socket.
         */
        NutTcpCloseSocket(sock);
    }
}
