/*!
 * Copyright (C) 2001-2002 by egnite Software GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *
 *    This product includes software developed by egnite Software GmbH
 *    and its contributors.
 *
 * THIS SOFTWARE IS PROVIDED BY EGNITE SOFTWARE GMBH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL EGNITE
 * SOFTWARE GMBH OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Log: inetq.c,v $
 * Revision 1.4  2002/06/12 10:55:54  harald
 * *** empty log message ***
 *
 * Revision 1.3  2002/06/04 19:11:29  harald
 * *** empty log message ***
 *
 * Revision 1.2  2002/05/08 16:02:30  harald
 * First Imagecraft compilation
 *
 */

/*!
 * \example inetq/inetq.c
 *
 * Requests an URL from the Internet. 
 * Demonstrates DNS query and default route usage.
 */
#define DNSSERVERIP     "192.168.192.2"
#define INETSERVER	    "www.google.com"
#define INETSERVERPORT  80
#define INETURL         "/search?q=ethernut"
#define MY_MAC          {0x00,0x06,0x98,0x20,0x00,0x00}
#define MY_IP           "192.168.192.100"
#define MY_MASK         "255.255.255.0"
#define MY_GATE         "192.168.192.3"

#include <string.h>

#include <dev/uartavr.h>
#include <dev/nicrtl.h>

#include <sys/heap.h>
#include <sys/thread.h>
#include <sys/timer.h>
#include <sys/print.h>
#include <sys/kprint.h>

#include <netinet/sostream.h>
#include <arpa/inet.h>
#include <net/route.h>
#include <netdb.h>

static u_char buff[1024];
static u_char my_mac[] = MY_MAC;

/*
 * Main application routine. 
 *
 * Nut/OS automatically calls this entry after initialization.
 */
THREAD(NutMain, arg)
{
    NUTDEVICE *uart0;
    u_long baud = 115200;
    TCPSOCKET *sock;
    NUTDEVICE *sostream;
    u_long rip;

    NutRegisterDevice(&devUart0, 0, 0);
    uart0 = NutDeviceOpen("uart0");
    NutDeviceIOCtl(uart0, UART_SETSPEED, &baud);
	NutKPrintString("\r\nInetQuery 1.0\r\n");

    /*
    * Register Realtek controller at address 8300 hex
    * and interrupt 5 and configure lan interface.
    */
    NutKPrintString("Configuring Ethernet interface\r\n");
    NutRegisterDevice(&devEth0, 0x8300, 5);
	{
	    u_long ip = inet_addr(MY_IP);
		u_long msk = inet_addr(MY_MASK);
        NutNetIfConfig("eth0", my_mac, ip, msk);
	}
    NutIpRouteAdd(0, 0, inet_addr(MY_GATE), &devEth0);
    NutDnsConfig(0, 0, inet_addr(DNSSERVERIP));

    if((rip = NutDnsGetHostByName(INETSERVER)) != 0) {
    
        /*
         * Create a socket.
         */
        if((sock = NutTcpCreateSocket()) != 0) {
            NutKPrintFormat("Connecting %s:%u\r\n", inet_ntoa(rip), INETSERVERPORT);
            if(NutTcpConnect(sock, rip, INETSERVERPORT) == 0) {
                if((sostream = NutSoStreamCreate(sock)) != 0) {
                    NutPrintFormat(sostream, "GET %s HTTP/1.0\r\n", INETURL);
                    NutPrintString(sostream, "User-Agent: Ethernut/2.3 [en] (NutOS)\r\n");
                    NutPrintString(sostream, "\r\n");
                    NutPrintFlush(sostream);
                    while(NutDeviceGetLine(sostream, buff, sizeof(buff) - 1) >= 0) {
                        NutKPrintString(buff);
                        NutKPrintString("\r\n");
                    }
                    NutSoStreamDestroy(sostream);
                }
                else
                    NutKPrintString("Creating stream device failed\r\n");

            }
            NutKPrintFormat("Disonnecting %s:%u\r\n", inet_ntoa(rip), INETSERVERPORT);
            NutTcpCloseSocket(sock);
        }
    }
    else
        NutKPrintFormat("Can't resolve %s\r\n", INETSERVER);

    for(;;)
        NutSleep(1000);
}

